/* screensize.cpp: Win NT utility to save & restore console
   window buffer size.  Parses command line for /save, /set,
   or /show arguments (optional & mutually exclusive).
   Written 7/13/94 by Frank Brown.
*/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <process.h>

#define DISPLAY 1
#define NODISPLAY 0

// Function prototypes:

void ErrExit(int,char *);
void GetXY(BOOL);
void ProcessCmdLine(int,char **);
void ReadFile(char *);
void SetXY();
void ShowHelp();
void WriteFile(char *);

// Global variables:

const char szAppName[] = "ScreenSize";
const char szVer[] = "1.0";
char szBuf[81],szMsg[81],szDefaultDataFile[81];
HANDLE hStdOut;
COORD dwWinSize;
CONSOLE_SCREEN_BUFFER_INFO csbi;
FILE *fp;
int i;

// print error message & exit
void ErrExit(int nStatus,char *szMsg)
{
printf("\nError: %s\n",szMsg);
exit(nStatus);
}

// get current window size
void GetXY(BOOL bShow)
{
if (!GetConsoleScreenBufferInfo(hStdOut,&csbi))
  ErrExit(1,"GetConsoleScreenBufferInfo");

dwWinSize = csbi.dwSize;

if (bShow)
  printf("\nX = %d, Y = %d\n",dwWinSize.X,dwWinSize.Y);
}

void main(int argc, char **argv)
{
sprintf(szDefaultDataFile,"%s\\CSBI$_.DAT",
            getenv("HOMEPATH"));

// get handle to stdout
hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
if (hStdOut == INVALID_HANDLE_VALUE)
  ErrExit(1,"Bad StdOut handle");

if (argc > 1)
  ProcessCmdLine(argc,argv);
else
  GetXY(DISPLAY);
}

// parse 1st arg
void ProcessCmdLine(int argc,char **argv)
{
char *pch;

if (*argv[1] == '/' || *argv[1] == '-') {
  ++argv[1];
  if (!strnicmp(argv[1],"sho",3))
    GetXY(DISPLAY);
  else if (!strnicmp(argv[1],"set",3)) {
    pch = strchr(argv[1],'=');
    if (!pch)                 // no values; use default file
      ReadFile(0);
    else {                    // parse what follows the '=':
      if (!strchr(pch,',')){  // no comma, must be filename
        ++pch;
        ReadFile(pch);
        }
      else {                  // comma found; scan values
        ++pch;                                
        sscanf(pch,"%3d,%3d",&dwWinSize.X,&dwWinSize.Y);  
        }
      }
    SetXY();
    }
  else if (!strnicmp(argv[1],"sav",3)) {
    GetXY(NODISPLAY);
    pch = strchr(argv[1],'=');
    if (pch)
      ++pch;
    WriteFile(pch);
    }
  else if (!strnicmp(argv[1],"hel",3) || *argv[1] == '?')
    ShowHelp();
  }
else {
  printf("\nUnknown argument %s",argv[1]);
  printf("\nType %s /help for information.\n", szAppName);
  }
}

// read window size settings from file
void ReadFile(char *szFName)
{
fp = fopen((szFName)? szFName : szDefaultDataFile,"r");
if (!fp) {
  sprintf(szMsg,"Problem opening file %s",(szFName)
        ? szFName : szDefaultDataFile);
  ErrExit(1,szMsg);
  }
if (!fgets(szBuf,7,fp))
  ErrExit(1,"fgets");
else
  sscanf(szBuf,"%3hd%3hd",&dwWinSize.X,&dwWinSize.Y);

fclose(fp);
}

// set current window size
void SetXY()
{
if (!SetConsoleScreenBufferSize(hStdOut,dwWinSize)) {
  sprintf(szMsg,"Bad value: X = %d, Y = %d\n",
            dwWinSize.X,dwWinSize.Y);
  ErrExit(1,szMsg);
  }
}

void ShowHelp()
{
printf("\n%s Version %s\n",szAppName,szVer);
puts("\nSave or set column and row values for an NT "
    "console window.\n");
printf("Usage: %s [/show | /set | /save | /help]\n",
    szAppName);
puts("\nAll arguments are optional and mutually "
    "exclusive:\n");
puts("/show\t\tDisplay current screen buffer values");
puts("/set\t\tSet screen buffer size to values stored "
    "in default file");
puts("/set=nnn,nnn\tSet screen buffer size to values "
    "(columns,rows)");
puts("/set=filename\tSet screen buffer size to values "
    "stored in filename");
puts("/save\t\tSave current screen buffer values to "
    "default file");
puts("/save=filename\tSave current screen buffer values "
    "in filename");
puts("/help\t\tDisplay this text\n");
}

void WriteFile(char *szFName)
{
fp = fopen((szFName)? szFName : szDefaultDataFile,"w");
if (!fp) {
  sprintf(szMsg,"Problem opening file %s",(szFName)
            ? szFName : szDefaultDataFile);
  ErrExit(1,szMsg);
  }
sprintf(szBuf,"%03d%03d",dwWinSize.X,dwWinSize.Y);
fputs(szBuf,fp);
fclose(fp);
}

