// wuistd.h - all WUIMAN source files include this header.
// Copyright (c) 1993 by Ron Burk

#ifndef WUISTD_H
#define WUISTD_H

#include <stdlib.h>

#undef ASSERT
#undef ASSERT2

extern void    AssertionFailure(char *File, int LineNumber,
                                                char *Text);
extern int  IsValidCodePtr(const void *);
extern int  IsValidDataPtr(const void *);
extern int  IsValidStringPtr(const char *);

#ifdef NDEBUG
    #define ASSERT(e)   ((void)0)
    #define ASSERT2(e)  (e)
#else
    #define ASSERT(e)   ((e) ? (void)0 : \
        AssertionFailure(__FILE__, __LINE__, #e))
    #define ASSERT2     ASSERT
#endif


#ifdef  WUI_ASSERT
    // work around GP fault in Symantec C++ v6.1
    #ifdef __SC__
        #define MEMBERASSERT() ASSERT(IsValidDataPtr((const void *)(this)))
        #define ASSERTCODEPTR(e) (IsValidCodePtr((const void*)e)? (void)0 : \
        AssertionFailure(__FILE__, __LINE__, #e))
        #define ASSERTDATAPTR(e) (IsValidDataPtr((const void*)e)? (void)0 : \
        AssertionFailure(__FILE__, __LINE__, #e))
        #define ASSERTSTRINGPTR(e) (IsValidStringPtr((const char*)e)? (void)0 : \
        AssertionFailure(__FILE__, __LINE__, #e))
    #else
    #define MEMBERASSERT() ASSERT(IsValidDataPtr((const void *)(this)))
    #define ASSERTCODEPTR(e) ASSERT(IsValidCodePtr((const void *)(e)))
    #define ASSERTDATAPTR(e) ASSERT(IsValidDataPtr((const void *)(e)))
    #define ASSERTSTRINGPTR(e) ASSERT(IsValidStringPtr((const char *)(e)))
    #endif
#else
    #define ASSERTCODEPTR(e) ((void)0)
    #define ASSERTDATAPTR(e) ((void)0)
    #define ASSERTSTRINGPTR(e) ((void)0)
    #define MEMBERASSERT() ((void)0)
#endif

// StringClone - allocate space for and duplicate a string.
char    *StringClone(const char *Input);

// StringCopy - Copies at most Max characters, last char copied
//              is always '\0'.
char    *StringCopy(char *To, const char *From, size_t Max);

// TCharBuffer - for heap character buffers too big for stack
class TCharBuffer
    {
public:
    TCharBuffer(size_t BufferSize);
    TCharBuffer(const TCharBuffer &Other);
    const TCharBuffer &operator=(const TCharBuffer &Other);
   ~TCharBuffer();
    operator char *();
//    operator char *() const;
    size_t  Size() const;
private:
    char    *Buffer;
    size_t  Size_;
    };

inline
TCharBuffer::operator char *()
    {   MEMBERASSERT();
    return Buffer;
    }

#if 0
inline
TCharBuffer::operator char *() const
    {   MEMBERASSERT();
    return Buffer;
    }
#endif

inline
size_t  TCharBuffer::Size() const
    {   MEMBERASSERT();
    return Size_;
    }


// provide separate debugging output
void    DebugPrintf(char Flag, const char *Format, ...);
void    DebugPrintf(const char *Format, ...);

#ifdef WUI_TRACE

#define DEBUG_MORE DebugPrintf
#define DEBUG_TRACE DebugPrintf('t', \
    "%-12s %04d ", __FILE__, __LINE__), DebugPrintf
#define DEBUG_WARNING DebugPrintf('w', \
    "%-12s %04d ", __FILE__, __LINE__), DebugPrintf
#define DEBUG_ERROR DebugPrintf('e', \
    "%-12s %04d ", __FILE__, __LINE__), DebugPrintf
#define DEBUG_FATAL DebugPrintf('f', \
    "%-12s %04d ", __FILE__, __LINE__), DebugPrintf

#else

#define DEBUG_MORE    (void)
#define DEBUG_TRACE   (void)
#define DEBUG_WARNING (void)
#define DEBUG_ERROR   (void)
#define DEBUG_FATAL   (void)

#endif  // WUI_TRACE



#ifndef TRUE
    #define TRUE    1
#endif
#ifndef FALSE
    #define FALSE   0
#endif



#endif
